;;
;;  To be run as follows:
;;
;;  scheme -no-init-file -compiler -heap 3000 -load <this file> -eval XXXX
;;
;;  where XXXX is (cf.edwin)     to compile edwin
;;                (cf.compiler)  to build compiler
;;                (cf.sf) (cf.cref) (cf.win32) (cf.6001)
;;

(define (with-error-report thunk)
  (bind-condition-handler (list condition-type:error)
      (lambda (condition)
	(display "\n\n\n\nBuild failed:\n\n")
	(write-condition-report condition (current-output-port))
	(display "\n\nType (fail) to quit Scheme.\n"))
    thunk))

(define (fail) (%exit 1))

(define (compile-dir checked? dir sf-er cbf-er)
  (with-error-report
      (lambda ()
	(with-working-directory-pathname dir
	  (lambda ()
	    (if checked?
		(begin
		  (display "\n\nCompiling with type checks and range checks ON")
		  (newline)
		  (set! compiler:generate-range-checks? #T)
		  (set! compiler:generate-type-checks? #T)))
	    (if sf-er (load sf-er))
	    (if cbf-er (load cbf-er))
	    (%exit))))))


(define (cf.compiler) (compile-dir #F "compiler"  "compiler.sf"  "compiler.cbf"))
(define (cf.cref)     (compile-dir #F "cref"      "cref.sf"      "cref.cbf"))
(define (cf.edwin)    (compile-dir #F "edwin"     "edwin.sf"     "edwin.cbf"))
(define (cf.runtime)  (compile-dir #F "runtime"   "runtime.sf"   "runtime.cbf"))
(define (cf.edwin/checked)   (compile-dir #T "ed-check" #F "edwin.cbf"))
(define (cf.runtime/checked) (compile-dir #T "rt-check" #F "runtime.cbf"))
(define (cf.sf)       (compile-dir #F "sf"        "sf.sf"        "sf.cbf"))
(define (cf.win32)    (compile-dir #F "win32"     "win32.sf"     "win32.cbf"))
(define (cf.6001)     (compile-dir #T "6001"      "6001.sf"      "6001.cbf"))





