(define install-group

  (in-package (->environment 'win32)

    (define get-version
      (windows-procedure (get-version) dword kernel32.dll "GetVersion"))

    (define variant 
      (let ((v (get-version)))
	(cond  ((> v 0) 'NT)
	       ((= 3 (remainder (+ v #x80000000) 256)) 'WINDOWS-3.1)
	       (else 'WINDOWS-95))))

    (define get-module-file-name
      (windows-procedure (get-module-file-name (module handle)
					       (name string)
					       (strlen int))
	int kernel32.dll "GetModuleFileNameA"))

    ;; figure out the binary directory from the module filename of the
    ;; executable
    (define binary-directory-pathname
      (let* ((buf     (make-string 256))
	     (hmodule (get-module-handle "SCHEME.EXE"))
	     (len     (get-module-file-name hmodule buf 256)))
	(directory-pathname (substring buf 0 len))))

    (define binary-directory
      (->namestring binary-directory-pathname))

    (define (sibling-directory pathname sibling-name)
      (->namestring
       (pathname-new-directory
	binary-directory-pathname
	(append (except-last-pair (pathname-directory pathname))
		(list sibling-name)))))
    (define lib-directory (sibling-directory binary-directory-pathname "lib"))
    (define doc-directory (sibling-directory binary-directory-pathname "doc"))
      
    (define pmgrpapi.dll (find-module "pmgrpapi.dll"))

    (define send-program-manager-command
      (windows-procedure (send-program-manager-command (cmd string))
	bool pmgrpapi.dll "SendProgramManagerCommand"))

    (define (spmc . args)
      (let ((dde-command (apply string-append args)))
	;;(display "\n;; ")
	;;(display dde-command)
	(send-program-manager-command dde-command)))

    (define (scheme-command . args)
      (apply string-append binary-directory "scheme.exe"
	     (if (null? args) "" (string-append " -library " lib-directory))
	     args))

    (define (bchscheme-command . args)
      (apply string-append binary-directory "bchschem.exe"	
	     (if (null? args) "" (string-append " -library " lib-directory))
	     args))

    (define (winhelp-command . args)  (apply string-append "winhelp" args))

    (define icons
      '(SHIELD3_ICON SHIELD4_ICON SHIELD2_ICON SHIELD1_ICON 
		     LAMBDA_ICON LAMBDA2_ICON EDWIN_ICON LIAR1_ICON
		     LIAR2_ICON LIAR3_ICON GRAPHICS_ICON CONSES_ICON
		     ENVIRONMENT_ICON COFFEE_ICON MINCER_ICON BCH_ICO))

    (define (position item seq)
      (if (eq? item (car seq))
	  0
	  (1+ (position item (cdr seq)))))

    (define (add-item command caption icon-file icon . others)
      (define icon-index
	(if (symbol? icon)
	    (position icon icons)
	    icon)) 
      (define (add/nt)
	(spmc "[AddItem(" 
	      "\"" command "\","  
	      "\"" caption "\","
	      "\"" icon-file "\","
	      (number->string icon-index)
	      ",10000,10000,%HOMEPATH%"
	      ")]"))
      (define (add/windows-95)
	(define (quotify s)
	  (string-replace (string-replace s #\( #\{) #\) #\}))
	(spmc "[AddItem(" 
	      (quotify command) ","
	      "\"" caption "\","
	      "" icon-file ","
	      (number->string icon-index)
	      ",10000,10000"   ;;   ",10000,10000,%HOMEPATH%"
	      ")]"))
      (define (add/win32s)
	(spmc "[AddItem(" 
	      "\"" command "\","  
	      "\"" caption "\","
	      "\"" icon-file "\","
	      (number->string icon-index)
	      ")]"))

      (display (string-append "\nNew Item"
			      "\n  Description:  " caption
			      "\n  Command Line: " command
			      ))
      (case variant
	((NT)          (add/nt))
	((WINDOWS-3.1) (add/win32s))
	((WINDOWS-95)  (add/windows-95))))

    (define (go)
      (if (memq variant '(windows-3.1 windows-95))
	  (set! icons (sort icons symbol<?)))
      (define group "MIT Scheme 7.4.2")
      (spmc "[CreateGroup("  "\""  group  "\""  ")]")

      (add-item (scheme-command "")
		"Runtime"
		(scheme-command) 'LAMBDA_ICON)

      (add-item (scheme-command " -large -band all.com -eval (edit)")
		"MIT Scheme (Edwin+Compiler)"
		(scheme-command) 'SHIELD3_ICON)

      (add-item (scheme-command " -edwin -eval \(edit\)")
		"Edwin"
		(scheme-command) 'EDWIN_ICON)

      (add-item (scheme-command " -compiler")
		"Compiler"
		(scheme-command) 'LIAR1_ICON)

      (add-item (bchscheme-command " -large -band all.com -eval (edit)")
		"BchScheme"
		(bchscheme-command) 'BCH_ICO)

      (add-item (winhelp-command " " doc-directory  "scheme.hlp")
		"MIT Scheme Reference Manual"
		"" 0)

      (add-item (winhelp-command " " doc-directory  "user.hlp")
		"MIT Scheme Users Manual"
		"" 0)

      (add-item (winhelp-command " " doc-directory  "r4rs.hlp")
		"R4RS"
		"" 0)

      (define (band-builder band)
	(scheme-command
	 " -large -no-init-file -load "
	 (->namestring (system-library-pathname "../etc/build.scm"))
	 " -eval (" band ")"
	 ))

      (if (eq? variant 'WINDOWS-3.1)
	  (begin
	    (add-item (band-builder "edwin.com")
		      "Build EDWIN.COM band"
		      (scheme-command) 'MINCER_ICON)
	    (add-item (band-builder "compiler.com")
		      "Build COMPILER.COM band"
		      (scheme-command) 'MINCER_ICON)
	    (add-item (band-builder "all.com")
		      "Build ALL.COM band"
		      (scheme-command) 'MINCER_ICON)))
      )

    go))

(install-group)
(%exit)
