(define install-group

  (in-package (->environment 'win32)

    (define get-version
      (windows-procedure (get-version) dword kernel32.dll "GetVersion"))

    (define nt? (> (get-version) 0))
    (define win32s? (not nt?))

    (define get-module-file-name
      (windows-procedure (get-module-file-name (module handle)
					       (name string)
					       (strlen int))
	int kernel32.dll "GetModuleFileNameA"))

    ;; figure out the binary directory from the module filename of the
    ;; executable
    (define binary-directory
      (let* ((buf     (make-string 256))
	     (hmodule (get-module-handle "SCHEME.EXE"))
	     (len     (get-module-file-name hmodule buf 256)))
	(->namestring (directory-pathname (substring buf 0 len)))))

    (define pmgrpapi.dll (find-module "pmgrpapi.dll"))

    (define send-program-manager-command
      (windows-procedure (send-program-manager-command (cmd string))
	bool pmgrpapi.dll "SendProgramManagerCommand"))

    (define (spmc . args)
      (let ((dde-command (apply string-append args)))
	;;(display "\n;; ")
	;;(display dde-command)
	(send-program-manager-command dde-command)))

    (define (scheme-command . args)
      (apply string-append binary-directory "scheme.exe" args))

    (define (bchscheme-command . args)
      (apply string-append binary-directory "bchschem.exe" args))

    (define (winhelp-command . args)  (apply string-append "winhelp" args))

    (define icons
      '(SHIELD3_ICON SHIELD4_ICON SHIELD2_ICON SHIELD1_ICON 
		     LAMBDA_ICON LAMBDA2_ICON EDWIN_ICON LAIR1_ICON
		     LAIR2_ICON LAIR3_ICON GRAPHICS_ICON CONSES_ICON
		     ENVIRONMENT_ICON MINCER_ICON BCH_ICO))

    (define (position item seq)
      (if (eq? item (car seq))
	  0
	  (1+ (position item (cdr seq)))))

    (define (add-item command caption icon-file icon . others)
      (define icon-index
	(if (symbol? icon)
	    (position icon icons)
	    icon))
      (display (string-append "\n\nNew Item"
			      "\n  Description:  " caption
			      "\n  Command Line: " command
			      ))
      (cond (nt?
	     (spmc "[AddItem(" 
		   "\"" command "\","  
		   "\"" caption "\","
		   "\"" icon-file "\","
		   (number->string icon-index)
		   ",10000,10000,%HOMEPATH%"
		   ")]"))
	    (win32s?
	     (spmc "[AddItem(" 
		   "\"" command "\","  
		   "\"" caption "\","
		   "\"" icon-file "\","
		   (number->string icon-index)
		   ")]"))))

    (define (go)
      (if win32s? (set! icons (sort icons symbol<?)))
      (define group "MIT Scheme Test Group")
      (spmc "[CreateGroup("  "\""  group  "\""  ")]")

      (add-item (scheme-command "")
		"Runtime"
		(scheme-command) 'LAMBDA_ICON)

      (add-item (scheme-command " -large -band all.com -eval (edit)")
		"MIT Scheme (Edwin+Compiler)"
		(scheme-command) 'SHIELD3_ICON)

      (add-item (scheme-command " -edwin -eval (edit)")
		"Edwin"
		(scheme-command) 'EDWIN_ICON)

      (add-item (scheme-command " -compiler")
		"Compiler"
		(scheme-command) 'LAIR1_ICON)

      (add-item (bchscheme-command " -large -band all.com -eval (edit)")
		"BchScheme"
		(bchscheme-command) 'BCH_ICO)

      (add-item (winhelp-command " " binary-directory  "scheme.hlp")
		"MIT Scheme Reference Manual"
		"" 0)

      (add-item (winhelp-command " " binary-directory  "user.hlp")
		"MIT Scheme Users Manual"
		"" 0)

      (define (band-builder band)
	(scheme-command
	 " -large -no-init-file -load "
	 (->namestring (system-library-pathname "../etc/build.scm"))
	 " -eval (" band ")"))

      (add-item (band-builder "edwin.com")
		"Build EDWIN.COM band"
		(scheme-command) 'MINCER_ICON)
      (add-item (band-builder "compiler.com")
		"Build COMPILER.COM band"
		(scheme-command) 'MINCER_ICON)
      (add-item (band-builder "all.com")
		"Build ALL.COM band"
		(scheme-command) 'MINCER_ICON)
      )

    go))


(install-group)
(%exit)
